#!/usr/bin/env python
"""Django's command-line utility for administrative tasks."""
import sys
import os
# print(os.path.dirname(sys.path[0]))
# print("/Users/yangzhengyuan/work/my/fast/code/fast-tracker-sw-python/venv/lib/python3.8")
# exit()



sys.path.append("/Users/yangzhengyuan/work/my/fast/code/fast-tracker-sw-python")
# sys.path.append(os.path.dirname(sys.path[0]))
# if __name__ == '__main__':
#     print(os.path.dirname(sys.path[0]))
#     exit()
#     sys.path.append(os.path.dirname(sys.path[0]))

from fast_tracker import agent, config, Component
from fast_tracker.trace.tags import Tag

# print(config)
# exit()


# SW_AGENT_PROTOCOL = 'http'
config.init(collector='127.0.0.1:11800', service='your awesome service')
# config.init(collector='127.0.0.1:5140', service='your awesome service')
agent.start()
# print("1111111111111111")
# exit()

import os
import sys


'''
单个方法追踪模式：
from fast_tracker.decorators import trace

@trace()
'''
# from fast_tracker.decorators import trace
# from fast_tracker.trace.context import get_context
#
# @trace()
def main():
    # context: SpanContext = get_context()
    print("mmmmmmmmmmm")

    # with context.new_entry_span(op="https://github.com/apache") as span:
    #     span.component = Component.Django
    #     span.tag(Tag(key='Org', val="Apache"))
    #     print("xxxxxxxxxxxxxxxxx")
    # print("zzzzzzzzzzzzzzzzzzzzzzzz")




    """Run administrative tasks."""
    os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'testdj.settings')
    try:
        from django.core.management import execute_from_command_line
    except ImportError as exc:
        raise ImportError(
            "Couldn't import Django. Are you sure it's installed and "
            "available on your PYTHONPATH environment variable? Did you "
            "forget to activate a virtual environment?"
        ) from exc
    execute_from_command_line(sys.argv)


if __name__ == '__main__':
    main()



#!/usr/local/bin python3
# -*- coding: utf-8 -*-

"""
    created by FAST-DEV 2021/4/6
"""

from abc import ABC
from queue import Queue


class Protocol(ABC):
    def connected(self):
        raise NotImplementedError()

    def heartbeat(self):
        raise NotImplementedError()

    def report(self, queue: Queue, block: bool = True):
        raise NotImplementedError()



#!/usr/local/bin python3
# -*- coding: utf-8 -*-

"""
    created by FAST-DEV 2021/4/6
"""

import time
from collections import namedtuple
from enum import Enum
from typing import List

version = '3.0.63'
version_info = list(map(int, version.split('.')))


class Component(Enum):
    Unknown = 0
    General = 7000  # built-in modules that may not have a logo to display
    Flask = 7001
    Requests = 7002
    PyMysql = 7003
    Django = 7004
    Tornado = 7005
    Redis = 7
    MongoDB = 9
    KafkaProducer = 40
    KafkaConsumer = 41
    RabbitmqProducer = 52
    RabbitmqConsumer = 53
    Elasticsearch = 47
    Urllib3 = 7006
    Sanic = 7007
    AioHttp = 7008
    Pyramid = 7009


class ComponentType():
    Unknown = "Unknown"
    General = "General"  # built-in modules that may not have a logo to display
    Flask = "Flask"
    Falcon = "Falcon"
    Requests = "Requests"
    PyMysql = "PyMysql"
    Django = "Django"
    Tornado = "Tornado"
    Redis = "Redis"
    MongoDB = "MongoDB"
    Kafka = "Kafka"
    KafkaProducer = "KafkaProducer"
    KafkaConsumer = "KafkaConsumer"
    Rabbitmq = "Rabbitmq"
    RabbitmqProducer = "RabbitmqProducer"
    RabbitmqConsumer = "RabbitmqConsumer"
    Elasticsearch = "ElasticSearch"
    Urllib3 = "Urllib3"
    Sanic = "Sanic"
    AioHttp = "AioHttp"
    Pyramid = "Pyramid"
    CustomLog = "CustomLog"
    Log = "Log"


class Layer(Enum):
    Local = 0
    Database = 1
    RPCFramework = 2
    Http = 3
    MQ = 4
    Cache = 5
    User = 6
    Log = 7


class Kind(Enum):
    Local = 0
    Entry = 1
    Exit = 2

    @property
    def is_local(self):
        return self == Kind.Local

    @property
    def is_entry(self):
        return self == Kind.Entry

    @property
    def is_exit(self):
        return self == Kind.Exit


LogItem = namedtuple("LogItem", "key val")


class Log(object):
    def __init__(self, timestamp: time = time.time(), items: List[LogItem] = None):
        self.timestamp = timestamp
        self.items = items or []



#
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import inspect
import os
import time
from os.path import dirname
from typing import Callable

import pytest
from _pytest.fixtures import FixtureRequest
from testcontainers.compose import DockerCompose


@pytest.fixture
def version():
    # type: () -> str
    return ''


# noinspection PyUnusedLocal
@pytest.fixture
def prepare():
    # type: () -> Callable
    return lambda *_: None


@pytest.fixture
def docker_compose(request, prepare, version):
    # type: (FixtureRequest, Callable, str) -> None

    module = request.module
    cwd = dirname(inspect.getfile(module))

    if version:
        with open(os.path.join(cwd, 'requirements.txt'), mode='w') as req:
            req.write(version)

    compose = DockerCompose(filepath=cwd)

    compose.start()

    exception = None
    for _ in range(0, 10):
        try:
            prepare()
            exception = None
            break
        except Exception as e:
            time.sleep(10)
            exception = e
    if exception:
        compose.stop()
        raise Exception("""Wait time exceeded {0} sec. Exception {1}""".format(100, exception))

    yield compose

    compose.stop()



#!/usr/local/bin python3
# -*- coding: utf-8 -*-

"""
    created by FAST-DEV 2021/4/6
"""

from fast_tracker import Layer, ComponentType
from fast_tracker.trace import tags
from fast_tracker.trace.context import get_context
from fast_tracker.trace.tags import Tag
from fast_tracker.utils import functions


def install():
    from urllib3.request import RequestMethods

    _request = RequestMethods.request

    def _fast_request(this: RequestMethods, method, url, fields=None, headers=None, **urlopen_kw):

        from urllib.parse import urlparse

        # from urllib.parse import urlsplit
        # split_result=urlsplit("https://www.google.com.hk/search?q=python+url%3F+%E8%A7%A3%E6%9E%90%3F&newwindow=1&safe=strict&sxsrf=ALeKk00UWlOugflkW77YePhdTqe0wGavqA%3A1618295940827&ei=hDx1YLjzMder3AOd67a4Cw&oq=python+url%3F+%E8%A7%A3%E6%9E%90%3F&gs_lcp=Cgdnd3Mtd2l6EAMyBAgjECcyBAgAEB4yBAgAEB4yBggAEAgQHjoHCAAQRxCwA1C9Ili9ImCjJmgBcAJ4AIABpAKIAZgEkgEDMi0ymAEAoAEBqgEHZ3dzLXdpesgBCMABAQ&sclient=gws-wiz&ved=0ahUKEwi4xcaVzvrvAhXXFXcKHZ21DbcQ4dUDCA4&uact=5")
        # SplitResult(scheme='https', netloc='www.google.com.hk', path='/search', query='q=python+url%3F+%E8%A7%A3%E6%9E%90%3F&newwindow=1&safe=strict&sxsrf=ALeKk00UWlOugflkW77YePhdTqe0wGavqA%3A1618295940827&ei=hDx1YLjzMder3AOd67a4Cw&oq=python+url%3F+%E8%A7%A3%E6%9E%90%3F&gs_lcp=Cgdnd3Mtd2l6EAMyBAgjECcyBAgAEB4yBAgAEB4yBggAEAgQHjoHCAAQRxCwA1C9Ili9ImCjJmgBcAJ4AIABpAKIAZgEkgEDMi0ymAEAoAEBqgEHZ3dzLXdpesgBCMABAQ&sclient=gws-wiz&ved=0ahUKEwi4xcaVzvrvAhXXFXcKHZ21DbcQ4dUDCA4&uact=5', fragment='')
        url_param = urlparse(url)
        # ParseResult(scheme='https', netloc='www.google.com.hk', path='/search', params='', query='q=python+url%3F+%E8%A7%A3%E6%9E%90%3F&newwindow=1&safe=strict&sxsrf=ALeKk00UWlOugflkW77YePhdTqe0wGavqA%3A1618295940827&ei=hDx1YLjzMder3AOd67a4Cw&oq=python+url%3F+%E8%A7%A3%E6%9E%90%3F&gs_lcp=Cgdnd3Mtd2l6EAMyBAgjECcyBAgAEB4yBAgAEB4yBggAEAgQHjoHCAAQRxCwA1C9Ili9ImCjJmgBcAJ4AIABpAKIAZgEkgEDMi0ymAEAoAEBqgEHZ3dzLXdpesgBCMABAQ&sclient=gws-wiz&ved=0ahUKEwi4xcaVzvrvAhXXFXcKHZ21DbcQ4dUDCA4&uact=5', fragment='')
        context = get_context()
        with context.new_exit_span(op=url_param.path or "/", peer=url_param.netloc) as span:
            carrier = span.inject()
            span.layer = Layer.Http
            span.component = ComponentType.Urllib3

            if headers is None:
                headers = {}
            for item in carrier:
                headers[item.key] = item.val

            span.tag(Tag(key=tags.HttpMethod, val=method.upper()))
            span.tag(Tag(key=tags.HttpUrl, val=url))
            # HttpPath 只需要获取PATH_INFO信息即可，不需要？后的内容
            span.tag(Tag(key=tags.HttpPath, val=(url_param.path).split("?")[0]))

            res = _request(this, method, url, fields=fields, headers=headers, **urlopen_kw)

            span.tag(Tag(key=tags.HttpStatus, val=res.status, overridable=True))
            if res.status >= 400:
                span.error_occurred = True

            functions.log("urllib3 span is: %r", span)

            return res

    RequestMethods.request = _fast_request



#!/usr/local/bin python3
# -*- coding: utf-8 -*-

"""
    created by iprobeyang@gmail.com 2021/4/22
"""
import os
import sys
import time

startup_debug = os.environ.get("FAST_DEBUG", "off").lower() in ("on", "true", "1")
fast_debug_level = int(os.environ.get("FAST_DEBUG_LEVEL", "0"))


def log_message(text, *args, level=1):
    if startup_debug:
        if fast_debug_level >= level:
            text = text % args
            timestamp = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
            sys.stdout.write("FAST: %s (%d) - %s\n" % (timestamp, os.getpid(), text))
            sys.stdout.flush()


log_message("working_directory = %r", os.getcwd())

log_message("sys.prefix = %r", os.path.normpath(sys.prefix))

try:
    log_message("sys.real_prefix = %r", sys.real_prefix)
except AttributeError:
    pass

log_message("sys.version_info = %r", sys.version_info)
log_message("sys.executable = %r", sys.executable)

if hasattr(sys, "flags"):
    log_message("sys.flags = %r", sys.flags)

log_message("sys.path = %r", sys.path)

for name in sorted(os.environ.keys()):
    if name.startswith("FAST_") or name.startswith("PYTHON"):
        log_message("%s = %r", name, os.environ.get(name))

import imp

boot_directory = os.path.dirname(__file__)
root_directory = os.path.dirname(os.path.dirname(boot_directory))

log_message("root_directory = %r", root_directory)
log_message("boot_directory = %r", boot_directory)

path = list(sys.path)

if boot_directory in path:
    del path[path.index(boot_directory)]

try:
    (file, pathname, description) = imp.find_module("sitecustomize", path)
except ImportError:
    pass
else:
    log_message("sitecustomize = %r", (file, pathname, description))

    imp.load_module("sitecustomize", file, pathname, description)

expected_python_prefix = os.environ.get("FAST_PYTHON_PREFIX")
log_message("FAST_PYTHON_PREFIX= %r", os.environ.get("FAST_PYTHON_PREFIX"))
# sys.prefix = expected_python_prefix
actual_python_prefix = os.path.realpath(os.path.normpath(sys.prefix))
log_message("actual_python_prefix= %r", actual_python_prefix)

expected_python_version = os.environ.get("FAST_PYTHON_VERSION")
actual_python_version = ".".join(map(str, sys.version_info[:2]))

python_prefix_matches = expected_python_prefix == actual_python_prefix
python_version_matches = expected_python_version == actual_python_version

log_message("python_prefix_matches = %r", python_prefix_matches)
log_message("python_version_matches = %r", python_version_matches)

if python_prefix_matches and python_version_matches:

    # config_file = os.environ.get("FAST_CONFIG_FILE", None)
    config_file = os.environ.get("FastTracker_ConfigPath", None)
    environment = os.environ.get("FAST_ENVIRONMENT", None)

    log_message("initialize_agent = %r", bool(config_file))
    log_message("config_file_dir: %r", config_file)
    if config_file:
        do_insert_path = root_directory not in sys.path
        if do_insert_path:
            sys.path.insert(0, root_directory)

        import fast_tracker.config

        log_message("agent_version = %r", fast_tracker.version)

        if do_insert_path:
            try:
                del sys.path[sys.path.index(root_directory)]
            except Exception:
                pass

    log_message("agent will init", level=4)
    # 探针初始化
    try:
        # 解决gunicorn+tornado模式下数据上报阻塞问题
        from gevent import monkey

        monkey.patch_all()

        import grpc.experimental.gevent as grpc_gevent

        grpc_gevent.init_gevent()

        log_message("agent start import", level=4)
        from fast_tracker import agent, config

        log_message("agent init start", level=4)
        # config.init(collector="127.0.0.1:8111", service="as you wish")
        config.init()
        agent.start()
        log_message("agent init end", level=4)
    except BaseException as e:
        log_message("发生错误了，错误信息是: %s", e)
        print("发生错误了，错误信息是", e)



#
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from skywalking import config, agent

if __name__ == '__main__':
    config.service_name = 'consumer'
    config.logging_level = 'INFO'
    agent.start()

    topic = "skywalking"
    server_list = ["kafka-server:9092"]
    group_id = "skywalking"
    client_id = "0"

    from kafka import KafkaConsumer
    from kafka import TopicPartition
    consumer = KafkaConsumer(group_id=group_id,
                             client_id=client_id,
                             bootstrap_servers=server_list)
    partition = TopicPartition(topic, int(client_id))
    consumer.assign([partition])
    for msg in consumer:
        print(msg)



#!/usr/local/bin python3
# -*- coding: utf-8 -*-

"""
    created by FAST-DEV 2021/4/6
"""

from fast_tracker import Layer, ComponentType, config
from fast_tracker.trace import tags
from fast_tracker.trace.carrier import Carrier
from fast_tracker.trace.context import get_context
from fast_tracker.trace.span import NoopSpan
from fast_tracker.trace.tags import Tag
from fast_tracker.utils import functions
from fast_tracker.utils.reader_type import ReaderType


def install():
    from flask import Flask

    _full_dispatch_request = Flask.full_dispatch_request
    _handle_user_exception = Flask.handle_user_exception
    _handle_exception = Flask.handle_exception

    def params_tostring(params):
        return "\n".join([k + "=[" + ",".join(params.getlist(k)) + "]" for k, _ in params.items()])

    def _fast_full_dispatch_request(this: Flask):
        import flask

        req = flask.request
        context = get_context()
        carrier = Carrier()
        trace_id_name = config.get_trace_id_name()
        for item in carrier:
            if item.key.capitalize() in req.headers:
                item.val = req.headers[item.key.capitalize()]
            if trace_id_name.capitalize() in req.headers and type(item) is Carrier:
                item.set_frontend_trace_id(req.headers[trace_id_name.capitalize()])
        with context.new_entry_span(op=req.path, carrier=carrier) as span:
            span.layer = Layer.Http
            span.component = ComponentType.Flask
            span.peer = "%s:%s" % (req.environ["REMOTE_ADDR"], req.environ["REMOTE_PORT"])
            span.tag(Tag(key=tags.HttpMethod, val=req.method))
            span.tag(Tag(key=tags.HttpUrl, val=req.url.split("?")[0]))
            # HttpPath 只需要获取PATH_INFO信息即可，不需要？后的内容
            span.tag(Tag(key=tags.HttpPath, val=req.path))
            set_code(config.tenant_code_reader, req, "tenant_code")
            set_code(config.user_code_reader, req, "user_code")
            set_code(config.env_code_reader, req, "env_code")
            if req.values:
                span.tag(
                    Tag(key=tags.HttpParams, val=params_tostring(req.values)[0 : config.http_params_length_threshold])
                )
            resp = _full_dispatch_request(this)

            if resp.status_code >= 400:
                span.error_occurred = True

            span.tag(Tag(key=tags.HttpStatus, val=resp.status_code, overridable=True))
            functions.log("flask span is: %r", span)
            return resp

    def _fast_handle_user_exception(this: Flask, e):
        if e is not None:
            entry_span = get_context().active_span()
            if entry_span is not None and type(entry_span) is not NoopSpan:
                entry_span.raised()

        return _handle_user_exception(this, e)

    def _fast_handle_exception(this: Flask, e):
        if e is not None:
            entry_span = get_context().active_span()
            if entry_span is not None and type(entry_span) is not NoopSpan:
                entry_span.raised()

        return _handle_exception(this, e)

    Flask.full_dispatch_request = _fast_full_dispatch_request
    Flask.handle_user_exception = _fast_handle_user_exception
    Flask.handle_exception = _fast_handle_exception

    def set_code(reader, request, type="tenant_code"):
        """
        根据配置从不同渠道（cookie\header\querystring\env）获取tenant_code、user_code、env_code等数据
        :param reader:
        :param request:
        :param type:
        :return:
        """
        code = ""
        if not reader:
            return code

        reader_type = reader.get("ReaderType")

        if reader_type == ReaderType.Cookie:
            code = request.cookies.get(reader.get("ReaderKey"), "")
        elif reader_type == ReaderType.RequestHeader:
            code = request.headers[reader.get("ReaderKey")] if reader.get("ReaderKey") in request.headers else ""
        elif reader_type == ReaderType.QueryString:
            code = request.args.get(reader.get("ReaderKey"), "")
        elif reader_type == ReaderType.Environment:
            import os

            code = os.getenv(reader.get("ReaderKey"), "")
        else:
            code = ""

        if code:
            # 设置config的code，让其全局适用
            if type == "tenant_code":
                config.set_tenant_code(code)
            elif type == "user_code":
                config.set_user_code(code)
            elif type == "env_code":
                config.set_env_code(code)

        return code



#!/usr/local/bin python3
# -*- coding: utf-8 -*-

"""
    created by FAST-DEV 2021/4/6
"""

from typing import List

from fast_tracker import config
from fast_tracker.utils.lang import b64encode, b64decode


class CarrierItem(object):
    def __init__(self, key: str = "", val: str = ""):
        self.key = key  # type: str
        self.val = val  # type: str

    @property
    def key(self):
        return self.__key

    @key.setter
    def key(self, key: str):
        self.__key = key

    @property
    def val(self):
        return self.__val

    @val.setter
    def val(self, val: str):
        self.__val = val


class Carrier(CarrierItem):
    def __init__(
        self,
        trace_id: str = "",
        segment_id: str = "",
        span_id: str = "",
        service: str = "",
        service_instance: str = "",
        endpoint: str = "",
        client_address: str = "",
        correlation: dict = None,
    ):  # pyre-ignore
        tracker_name = config.get_tracker_name()
        super(Carrier, self).__init__(key=tracker_name)
        self.trace_id = trace_id  # type: str
        self.segment_id = segment_id  # type: str
        self.span_id = span_id  # type: str
        self.service = service  # type: str
        self.service_instance = service_instance  # type: str
        self.endpoint = endpoint  # type: str
        self.client_address = client_address  # type: str
        self.correlation_carrier = FastCorrelationCarrier()
        self.items = [self.correlation_carrier, self]  # type: List[CarrierItem]
        self.__iter_index = 0  # type: int
        if correlation is not None:
            self.correlation_carrier.correlation = correlation

    @property
    def val(self) -> str:
        return "-".join(
            [
                b64encode(self.trace_id),
                self.span_id,
                b64encode(self.segment_id),
                b64encode(self.service),
                b64encode(self.service_instance),
                b64encode(self.endpoint),
                b64encode(self.client_address),
            ]
        )

    @val.setter
    def val(self, val: str):
        self.__val = val
        if not val:
            return
        parts = val.split("-")
        if len(parts) != 7:
            return
        self.trace_id = b64decode(parts[0])
        self.span_id = parts[1]
        self.segment_id = b64decode(parts[2])
        self.service = b64decode(parts[3])
        self.service_instance = b64decode(parts[4])
        self.endpoint = b64decode(parts[5])
        self.client_address = b64decode(parts[6])

    def set_frontend_trace_id(self, trace_id):
        self.trace_id = trace_id

    @property
    def is_valid(self):
        # type: () -> bool
        return (
            # 修改其校验数据规则，因前端传入数据时，如x-fast-trace-id传入只会有trace_id而不会有其他内容
            len(self.trace_id) > 0
            # and len(self.segment_id) > 0
            # and len(self.service) > 0
            # and len(self.service_instance) > 0
            # and len(self.endpoint) > 0
            # and len(self.client_address) > 0
            # and self.span_id.isnumeric()
        )

    def __iter__(self):
        self.__iter_index = 0
        return self

    def __next__(self):
        if self.__iter_index >= len(self.items):
            raise StopIteration
        n = self.items[self.__iter_index]
        self.__iter_index += 1
        return n


class FastCorrelationCarrier(CarrierItem):
    def __init__(self):
        super(FastCorrelationCarrier, self).__init__(key="fast-correlation")
        self.correlation = {}  # type: dict

    @property
    def val(self) -> str:
        if self.correlation is None or len(self.correlation) == 0:
            return ""

        return ",".join([b64encode(k) + ":" + b64encode(v) for k, v in self.correlation.items()])

    @val.setter
    def val(self, val: str):
        self.__val = val
        if not val:
            return
        for per in val.split(","):
            if len(self.correlation) > config.correlation_element_max_number:
                break
            parts = per.split(":")
            if len(parts) != 2:
                continue
            self.correlation[b64decode(parts[0])] = b64decode(parts[1])



#!/usr/local/bin python3
# -*- coding: utf-8 -*-

"""
    created by iprobeyang@gmail.com 2021/4/20
"""

from fast_tracker import Layer, ComponentType, config
from fast_tracker.trace import tags
from fast_tracker.trace.carrier import Carrier
from fast_tracker.trace.context import get_context
from fast_tracker.trace.span import NoopSpan
from fast_tracker.trace.tags import Tag
from fast_tracker.utils.reader_type import ReaderType


def install():
    from falcon import API, request, response

    _original_falcon_api = API.__call__
    _original_falcon_handle_exception = API._handle_exception

    def params_tostring(params):
        return "\n".join([k + "=" + v for k, v in params.items()])

    def _fast_falcon_api(this: API, env, start_response):
        context = get_context()
        carrier = Carrier()
        headers = get_headers(env)
        trace_id_name = config.get_trace_id_name()
        for item in carrier:
            if item.key.capitalize() in headers:
                item.val = headers[item.key.capitalize()]
            if item.key.upper() in headers:
                item.val = headers[item.key.upper()]
            if trace_id_name.capitalize() in headers and type(item) is Carrier:
                item.set_frontend_trace_id(headers[trace_id_name.capitalize()])
            if trace_id_name.upper() in headers and type(item) is Carrier:
                item.set_frontend_trace_id(headers[trace_id_name.upper()])
        with context.new_entry_span(op="/", carrier=carrier) as span:
            span.layer = Layer.Http
            span.component = ComponentType.Falcon

            resp = _original_falcon_api(this, env, start_response)

            from falcon import RequestOptions
            from falcon import ResponseOptions

            req = request.Request(env, RequestOptions())
            resp_obj = response.Response(ResponseOptions())

            span.op = str(req.url).split("?")[0]
            span.peer = "%s:%s" % (req.remote_addr, req.port)
            # span.tag(Tag(key=tags.HttpMethod, val=req.method))
            # span.tag(Tag(key=tags.HttpUrl, val=str(req.url)))
            # span.tag(Tag(key=tags.HttpPath, val=str(req.url).split("?")[0]))
            if req.params:
                span.tag(
                    Tag(key=tags.HttpParams, val=params_tostring(req.params)[0 : config.http_params_length_threshold])
                )

            set_code(config.tenant_code_reader, req, "tenant_code")
            set_code(config.user_code_reader, req, "user_code")
            set_code(config.env_code_reader, req, "env_code")

            resp_status = parse_status(resp_obj.status)
            if int(resp_status[0]) >= 400:
                span.error_occurred = True

            return resp

    def _fast_handle_exception(this: API, req, resp, ex, params):
        print("a9")
        if ex is not None:
            entry_span = get_context().active_span()
            if entry_span is not None and type(entry_span) is not NoopSpan:
                entry_span.raised()

        return _original_falcon_handle_exception(this, req, resp, ex, params)

    def set_code(reader, request, type="tenant_code"):
        """
        根据配置从不同渠道（cookie\header\querystring\env）获取tenant_code、user_code、env_code等数据
        :param reader:
        :param request:
        :param type:
        :return:
        """
        code = ""
        if not reader:
            return code

        reader_type = reader.get("ReaderType")

        if reader_type == ReaderType.Cookie:
            code = request.cookies.get(reader.get("ReaderKey"), "")
        elif reader_type == ReaderType.RequestHeader:
            code = request.headers[reader.get("ReaderKey")] if reader.get("ReaderKey") in request.headers else ""
        elif reader_type == ReaderType.QueryString:
            # import json
            # raw = request.stream.read()
            raw_json = request.bounded_stream.read()
            from urllib import parse
            params = parse.parse_qsl(raw_json)
            if request.method == "GET":
                code = request.params.get(reader.get("ReaderKey"), "")
        elif reader_type == ReaderType.Environment:
            import os

            code = os.getenv(reader.get("ReaderKey"), "")
        else:
            code = ""

        if code:
            # 设置config的code，让其全局适用
            if type == "tenant_code":
                config.set_tenant_code(code)
            elif type == "user_code":
                config.set_user_code(code)
            elif type == "env_code":
                config.set_env_code(code)

        return code

    API.__call__ = _fast_falcon_api
    API._handle_exception = _fast_handle_exception


def get_headers(env):
    """
    获取headers数据
    :param dict env: 环境变量
    :return:
    """
    headers = {}
    wsgi_content_headers = frozenset(["CONTENT_TYPE", "CONTENT_LENGTH"])

    for name, value in env.items():
        if name.startswith("HTTP_"):
            headers[name[5:].replace("_", "-")] = value

        elif name in wsgi_content_headers:
            headers[name.replace("_", "-")] = value

    return headers


def parse_status(status_str):
    """
    解析falcon的response的status数据，其结构类似：200 OK,参考：python3.8/site-packages/falcon/status_codes.py
    :param status_str:
    :return:
    """
    return status_str.split(" ") if status_str else [404, "status is empty"]



#!/usr/local/bin python3
# -*- coding: utf-8 -*-

"""
    created by iprobeyang@gmail.com 2021/4/13
"""

from django.http import HttpResponse

from TestModel.models import Test


# 数据库操作
def testdb(request):
    test1 = Test(name='runoob')
    test1.save()
    return HttpResponse("<p>数据添加成功！</p>")


#!/usr/local/bin python3
# -*- coding: utf-8 -*-

"""
    created by FAST-DEV 2021/4/6
"""
import inspect
import os
import re
import uuid

# try:
#     import ConfigParser
# except ImportError:
#     import configparser as ConfigParser

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from typing import List

RE_IGNORE_PATH = os.getenv("FastTracker_ReIgnorePath") or re.compile("^$")  # type: re.Pattern

service_instance = os.getenv("FastTracker_Instance") or str(uuid.uuid1()).replace("-", "")  # type: str
protocol = (os.getenv("FastTracker_Protocol") or "udp").lower()  # type: str
authentication = os.getenv("FastTracker_Authentication")  # type: str
# logging_level = os.getenv("FastTrackerLoggingLevel") or "INFO"  # type: str
# 禁用插件列表 plugins init文件中会使用到，也可以使用FAST_TRACKER_AGENT_DISABLE_PLUGINS申明禁用
disable_plugins = (os.getenv("FastTracker_DisablePlugins") or "").split(",")  # type: List[str]
ignore_suffix = (
    os.getenv("FastTracker_IgnoreSuffix") or ".jpg,.jpeg,.js,.css,.png,.bmp,.gif,.ico,.mp3," ".mp4,.html,.svg "
)  # type: str
http_params_length_threshold = int(os.getenv("FastTracker_HttpParamsLengthThreshold") or "1024")  # type: int
django_collect_http_params = (
    True
    if os.getenv("FastTracker_DjangoCollectHttpParams") and os.getenv("FastTracker_DjangoCollectHttpParams") == "True"
    else False
)  # type: bool
correlation_element_max_number = int(os.getenv("FastTracker_CorrelationElementMaxNumber") or "3")  # type: int
correlation_value_max_length = int(os.getenv("FastTracker_CorrelationValueMaxLength") or "128")  # type: int
trace_ignore_path = os.getenv("FastTracker_IgnorePath") or ""  # type: str

############################# fast_tracker默认配置信息开始 ###############################
# agent全局开关，默认false
enable = True if os.getenv("FastTracker_Enable") and os.getenv("FastTracker_Enable") == "True" else False  # type: bool
# 环境编码 默认test
env_code = os.getenv("FastTracker_EnvCode") or "test"  # type: str
# 日志等级 默认 DEBUG
# log_level = os.getenv("FastTrackerLogLevel") or "DEBUG"  # type: str
log_level = os.getenv("FastTracker_LogLevel") or "ERROR"  # type: str
# 租户编码 默认zsdc
tenant_code = os.getenv("FastTracker_TenantCode") or "zsdc"  # type: str
# 用户编码 默认zhangsan
user_code = os.getenv("FastTracker_UserCode") or "zhangsan"  # type: str
# 服务名称 默认"天眼平台"
service_name = os.getenv("FastTracker_ServiceName") or "天眼平台"  # type: str
# 产品编码 默认fast
product_code = os.getenv("FastTracker_ProductCode") or "fast"  # type: str
# 服务编码 默认"analysis"
app_code = os.getenv("FastTracker_AppCode") or "analysis"  # type: str
# 上报地址 默认"udp://127.0.0.1:5140"
collector_address = os.getenv("FastTracker_SocketPath") or "udp://fluent-bit:5140"  # type: str
# 缓冲区大小 默认1
buffer_size = os.getenv("FastTracker_BufferSize") or 1  # type: int
# 上报超时时间 默认1（秒）
socket_timeout = os.getenv("FastTracker_SocketTimeout") or 1  # type: int
# 兼容原Event模块，目前一般只有Components
event = {"Components": {"SqlClient": True, "HttpClient": True}}
# 租户code，一般SaaS化租户会在运行时才知道此值，所以动态传值，ReaderType表示从哪里获取，ReaderKey表示字段名称
# tenant_code_reader = {"ReaderType": "Cookie", "ReaderKey": "tenant_code"}
# tenant_code_reader = {"ReaderType": "RequestHeader", "ReaderKey": "tcode"}
tenant_code_reader = {"ReaderType": "QueryString", "ReaderKey": "tentcode"}
# 用户code，一般会在运行时才知道此值，所以动态传值，ReaderType表示从哪里获取，ReaderKey表示字段名称
user_code_reader = {"ReaderType": "Cookie", "ReaderKey": "user_code"}
# 环境code，一般会在运行时才知道此值，所以动态传值，ReaderType表示从哪里获取，ReaderKey表示字段名称
env_code_reader = {"ReaderType": "Cookie", "ReaderKey": "env_code"}
# 前端Header
carrier_header = {"TrackerName": "fast-tracker", "TraceIdName": "x-fast-trace-id"}
############################# fast_tracker默认配置信息结束 ###############################


def init(
    service: str = None,
    instance: str = None,
    collector: str = None,
    protocol_type: str = "udp",
    token: str = None,
):
    # 根据环境变量 FastTracker.ConfigPath 加载配置文件
    config_file = os.environ.get("FastTracker_ConfigPath", None)
    print("FastTracker_ConfigPath:", config_file)
    if config_file:
        # _load_configuration(config_file)
        from fast_tracker.fast_tracker_configer import FastTrackerConfiger

        FastTrackerConfiger.load_configuration(config_file)

    global service_name
    service_name = service or service_name

    global service_instance
    service_instance = instance or service_instance

    global collector_address
    collector_address = collector or collector_address

    global protocol
    protocol = protocol_type or protocol

    global authentication
    authentication = token or authentication


def set_enable(cus_enable: bool = False):
    """
    设置enable值
    :param cus_enable:
    :return:
    """
    global enable
    enable = cus_enable


def set_env_code(cus_env_code: str = ""):
    """
    设置env_code值
    :param cus_env_code:
    :return:
    """
    global env_code
    env_code = cus_env_code


def set_tenant_code(cus_tenant_code: str = ""):
    """
    设置tenant_code值
    :param cus_tenant_code:
    :return:
    """
    global tenant_code
    tenant_code = cus_tenant_code


def set_user_code(cus_user_code: str = ""):
    """
    设置user_code值
    :param cus_user_code:
    :return:
    """
    global user_code
    user_code = cus_user_code


def set_product_code(cus_product_code: str = ""):
    """
    设置 product_code 值
    :param cus_product_code:
    :return:
    """
    global product_code
    product_code = cus_product_code


def set_app_code(cus_app_code: str = ""):
    """
    设置 app_code 值
    :param cus_app_code:
    :return:
    """
    global app_code
    app_code = cus_app_code


def set_service_name(cus_service_name: str = ""):
    """
    设置service_name值
    :param cus_service_name:
    :return:
    """
    global service_name
    service_name = cus_service_name


def set_socket_path(cus_collector_address: str = ""):
    """
    设置collector_address值
    :param cus_collector_address:
    :return:
    """
    global collector_address
    collector_address = cus_collector_address


def set_buffer_size(cus_buffer_size: int = 1):
    """
    设置buffer_size值
    :param cus_buffer_size:
    :return:
    """
    global buffer_size
    buffer_size = cus_buffer_size


def set_socket_timeout(cus_socket_timeout: int = 1):
    """
    设置socket_timeout值
    :param cus_socket_timeout:
    :return:
    """
    global socket_timeout
    socket_timeout = cus_socket_timeout


def set_event(cus_event: dict):
    """
    设置event值
    :param cus_event:
    :return:
    """
    global event
    if not cus_event:
        return False
    event = cus_event


def set_tenant_code_reader(cus_tenant_code_reader: dict):
    """
    设置tenant_code_reader值
    :param cus_tenant_code_reader:
    :return:
    """
    global tenant_code_reader
    if not cus_tenant_code_reader:
        return False
    tenant_code_reader = cus_tenant_code_reader


def set_user_code_reader(cus_user_code_reader: dict):
    """
    设置user_code_reader值
    :param cus_user_code_reader:
    :return:
    """
    global user_code_reader
    if not cus_user_code_reader:
        return False
    user_code_reader = cus_user_code_reader


def set_carrier_header(cus_carrier_header: dict):
    """
    设置carrier_header值
    :param cus_carrier_header:
    :return:
    """
    global carrier_header
    if not cus_carrier_header:
        return False
    carrier_header = cus_carrier_header


def set_trace_id_name(trace_id_name: str):
    """
    设置trace_id_name
    :param str trace_id_name:
    :return:
    """
    global carrier_header
    if not trace_id_name:
        return False
    carrier_header["TraceIdName"] = trace_id_name


def get_trace_id_name():
    """
    获取trace_id_name
    :return str
    """
    global carrier_header
    return carrier_header.get("TraceIdName")


def set_tracker_name(tracker_name):
    """
    设置 tracker_name
    :param tracker_name:
    :return:
    """
    global carrier_header
    if not tracker_name:
        return False
    carrier_header["TrackerName"] = tracker_name


def get_tracker_name():
    """
    获取 tracker_name
    :return str
    """
    global carrier_header
    return carrier_header.get("TrackerName")


def finalize():
    """
    通过忽略后缀的文件找到忽略的文件夹
    :return:
    """
    reesc = re.compile(r"([.*+?^=!:${}()|\[\]\\])")
    suffix = r"^.+(?:" + "|".join(reesc.sub(r"\\\1", s.strip()) for s in ignore_suffix.split(",")) + ")$"
    path = (
        "^(?:"
        + "|".join(  # replaces ","
            "(?:(?:[^/]+/)*[^/]+)?".join(  # replaces "**"
                "[^/]*".join(  # replaces "*"
                    "[^/]".join(reesc.sub(r"\\\1", s) for s in p2.split("?")) for p2 in p1.split("*")  # replaces "?"
                )
                for p1 in p0.strip().split("**")
            )
            for p0 in trace_ignore_path.split(",")
        )
        + ")$"
    )

    global RE_IGNORE_PATH
    RE_IGNORE_PATH = re.compile("%s|%s" % (suffix, path))


def serialize():
    from fast_tracker import config

    return {
        key: value
        for key, value in config.__dict__.items()
        if not (
            key.startswith("_")
            or key == "TYPE_CHECKING"
            or key == "RE_IGNORE_PATH"
            or inspect.isfunction(value)
            or inspect.ismodule(value)
            or inspect.isbuiltin(value)
            or inspect.isclass(value)
        )
    }


def deserialize(data):
    from fast_tracker import config

    for key, value in data.items():
        if key in config.__dict__:
            config.__dict__[key] = value
    finalize()



#!/usr/local/bin python3
# -*- coding: utf-8 -*-

"""
    created by FAST-DEV 2021/4/12
"""
from fast_tracker import config, Layer, ComponentType, Log, LogItem
from fast_tracker.trace.context import get_context
from fast_tracker.trace.tags import Tag


class FastTracker:
    def __init__(self):
        self.span = None

    @property
    def enable(self):
        """
        获取enable值
        :return:
        """
        return config.enable

    @enable.setter
    def enable(self, enable: bool = False):
        """
        设置enable值
        :param enable:
        :return:
        """
        config.set_enable(enable)

    @property
    def env_code(self):
        """
        获取 env_code 值
        :return:
        """
        return config.env_code

    @env_code.setter
    def env_code(self, env_code: str = ""):
        """
        设置 env_code 值
        :param env_code:
        :return:
        """
        config.set_env_code(env_code)

    @property
    def product_code(self):
        """
        获取 product_code 值
        :return:
        """
        return config.product_code

    @product_code.setter
    def product_code(self, product_code: str = ""):
        """
        设置 product_code 值
        :param product_code:
        :return:
        """
        config.set_product_code(product_code)

    @property
    def app_code(self):
        """
        获取 app_code 值
        :return:
        """
        return config.app_code

    @app_code.setter
    def app_code(self, app_code: str = ""):
        """
        设置 app_code 值
        :param app_code:
        :return:
        """
        config.set_app_code(app_code)

    @property
    def tenant_code(self):
        """
        获取 tenant_code 值
        :return:
        """
        return config.tenant_code

    @tenant_code.setter
    def tenant_code(self, tenant_code: str = ""):
        """
        设置 tenant_code 值
        :param tenant_code:
        :return:
        """
        config.set_tenant_code(tenant_code)

    @property
    def user_code(self):
        """
        获取 user_code 值
        :return:
        """
        return config.user_code

    @user_code.setter
    def user_code(self, user_code: str = ""):
        """
        设置 user_code 值
        :param user_code:
        :return:
        """
        config.set_user_code(user_code)

    @property
    def service_name(self):
        """
        获取 service_name 值
        :return:
        """
        return config.user_code

    @service_name.setter
    def service_name(self, service_name: str = ""):
        """
        设置 service_name 值
        :param service_name:
        :return:
        """
        config.set_service_name(service_name)

    @property
    def socket_path(self):
        """
        获取 socket_path 值
        :return:
        """
        return config.collector_address

    @socket_path.setter
    def socket_path(self, socket_path: str = ""):
        """
        设置 socket_path 值
        :param socket_path:
        :return:
        """
        config.set_socket_path(socket_path)

    @property
    def buffer_size(self):
        """
        获取 buffer_size 值
        :return:
        """
        return config.buffer_size

    @buffer_size.setter
    def buffer_size(self, buffer_size: int = 1):
        """
        设置 buffer_size 值
        :param buffer_size:
        :return:
        """
        config.set_buffer_size(buffer_size)

    @property
    def socket_timeout(self):
        """
        获取 socket_timeout 值
        :return:
        """
        return config.socket_timeout

    @socket_timeout.setter
    def socket_timeout(self, socket_timeout: int = 1):
        """
        设置 socket_timeout 值
        :param socket_timeout:
        :return:
        """
        config.set_socket_timeout(socket_timeout)

    @property
    def socket_timeout(self):
        """
        获取 socket_timeout 值
        :return:
        """
        return config.socket_timeout

    @socket_timeout.setter
    def socket_timeout(self, socket_timeout: int = 1):
        """
        设置 socket_timeout 值
        :param socket_timeout:
        :return:
        """
        config.set_socket_timeout(socket_timeout)

    @property
    def event(self):
        """
        获取 event 值
        :return:
        """
        return config.event

    @event.setter
    def event(self, event: dict):
        """
        设置 event 值
        :param event:
        :return:
        """
        config.set_event(event)

    @staticmethod
    def get_config():
        return {
            "ServiceName": config.service_name,
            "ServiceInstance": config.service_instance,
            "Protocol": config.protocol,
            "Authentication": config.authentication,
            "LogLevel": config.log_level,
            "DisablePlugins": config.disable_plugins,
            "IgnoreSuffix": config.ignore_suffix,
            "HttpParamsLengthThreshold": config.http_params_length_threshold,
            "DjangoCollectHttpParams": config.django_collect_http_params,
            "CorrelationElementMaxNumber": config.correlation_element_max_number,
            "CorrelationValueMaxLength": config.correlation_value_max_length,
            "TraceIgnorePath": config.trace_ignore_path,
            "Enable": config.enable,
            "EnvCode": config.env_code,
            "TenantCode": config.tenant_code,
            "UserCode": config.user_code,
            "SocketPath": config.collector_address,
            "BufferSize": config.buffer_size,
            "SocketTimeout": config.socket_timeout,
            "Event": config.event,
            "TenantCodeReader": config.tenant_code_reader,
            "UserCodeReader": config.user_code_reader,
            "CarrierHeader": config.carrier_header,
        }

    def begin_log(self):
        """
        log模块开始 类似数据库事务的begin_trasaction
        :return:
        """
        context = get_context()
        span = context.active_span()
        self.span = context.new_local_span(op="execute")
        self.span.layer = Layer.Local
        self.span.component = ComponentType.CustomLog
        self.span.pid = span.sid
        self.span.start()
        return self
        # # with 语法后不需要start和stop 普通模式需要start和stop
        # with context.new_local_span(op="execute") as self.span:
        #     self.span.layer = Layer.Local
        #     self.span.component = ComponentType.CustomLog
        #     self.span.pid = span.sid
        #     # self.span.start()
        #
        #     return self

    def debug(self, msg: str = ""):
        if not self.span:
            raise RuntimeError("请先调用begin_log实例化log对象")
        self.span.logs.append(Log(items=[LogItem(key="Debug", val=self.log_covert("Debug", msg))]))

    def info(self, msg: str = ""):
        if not self.span:
            raise RuntimeError("请先调用begin_log实例化log对象")
        self.span.logs.append(Log(items=[LogItem(key="Info", val=self.log_covert("Info", msg))]))

    def warning(self, msg: str = ""):
        if not self.span:
            raise RuntimeError("请先调用begin_log实例化log对象")
        self.span.logs.append(Log(items=[LogItem(key="Warning", val=self.log_covert("Warning", msg))]))

    def error(self, msg: str = ""):
        if not self.span:
            raise RuntimeError("请先调用begin_log实例化log对象")
        self.span.logs.append(Log(items=[LogItem(key="Error", val=self.log_covert("Error", msg))]))

    def critical(self, msg: str = ""):
        if not self.span:
            raise RuntimeError("请先调用begin_log实例化log对象")
        self.span.logs.append(Log(items=[LogItem(key="Critical", val=self.log_covert("Critical", msg))]))

    def log(self, msg: str = "", level: str = ""):
        if not self.span:
            raise RuntimeError("请先调用begin_log实例化log对象")
        self.span.logs.append(Log(items=[LogItem(key=level.capitalize(), val=self.log_covert(level, msg))]))

    def log_covert(self, level: str = "Debug", msg: str = ""):
        return {"err_msg": msg, "err_type": level, "err_trace": ""}

    def end_log(self):
        """
        log模块结束
        :return:
        """
        if not self.span:
            raise RuntimeError("请先调用begin_log实例化log对象")
        # with 语法后不需要start和stop 普通模式需要start和stop，取决于begin_log
        self.span.stop()



#!/usr/local/bin python3
# -*- coding: utf-8 -*-

"""
    created by FAST-DEV 2021/4/6
"""

import time
import traceback
from abc import ABC
from copy import deepcopy
from typing import List
from typing import TYPE_CHECKING

from fast_tracker import Kind, Layer, Log, Component, LogItem, config
from fast_tracker.trace import ID
from fast_tracker.trace.carrier import Carrier
from fast_tracker.trace.segment import SegmentRef, Segment
from fast_tracker.trace.tags import Tag
from fast_tracker.utils.lang import tostring

if TYPE_CHECKING:
    from fast_tracker.trace.context import SpanContext


@tostring
class Span(ABC):
    def __init__(
        self,
        context: "SpanContext",
        sid: str = "",
        pid: str = "",
        op: str = None,
        peer: str = None,
        kind: Kind = None,
        component: Component = None,
        layer: Layer = None,
    ):
        self.context = context  # type: SpanContext
        self.sid = sid  # type: str
        self.pid = pid  # type: str
        self.op = op  # type: str
        self.peer = peer  # type: str
        self.kind = kind  # type: Kind
        self.component = component or Component.Unknown  # type: Component
        self.layer = layer or Layer.Local  # type: Layer

        self.tags = []  # type: List[Tag]
        self.logs = []  # type: List[Log]
        self.refs = []  # type: List[SegmentRef]
        self.start_time = 0  # type: int
        self.end_time = 0  # type: int
        self.error_occurred = False  # type: bool

    def start(self):
        self.start_time = int(time.time() * 1000)
        self.context.start(self)

    def stop(self):
        return self.context.stop(self)

    def finish(self, segment: "Segment") -> bool:
        self.end_time = int(time.time() * 1000)
        segment.archive(self)
        return True

    def raised(self) -> "Span":
        self.error_occurred = True
        self.logs = [
            Log(
                items=[
                    LogItem(key="Traceback", val=self.format_exc()),
                ]
            )
        ]
        return self

    def format_exception(self, etype, value, tb, limit=None, chain=True):
        """Format a stack trace and the exception information.

        The arguments have the same meaning as the corresponding arguments
        to print_exception().  The return value is a list of strings, each
        ending in a newline and some containing internal newlines.  When
        these lines are concatenated and printed, exactly the same text is
        printed as does print_exception().
        """
        # format_exception has ignored etype for some time, and code such as cgitb
        # passes in bogus values as a result. For compatibility with such code we
        # ignore it here (rather than in the new TracebackException API).
        # 重写了python的traceback.py中的两个方法：format_exception、format_exc，但并未覆盖这两个方法
        data = traceback.TracebackException(type(value), value, tb, limit=limit)
        err = data._str
        err_log = data.stack[-1]
        trace = {
            "filename": err_log.filename,
            "line": err_log.line,
            "lineno": err_log.lineno,
            "func_name": err_log.name,
        }
        err_type = str(data.exc_type)
        return {"err_msg": err, "err_type": err_type, "err_trace": trace}

    def format_exc(self, limit=None, chain=True):
        import sys

        # self.format_exception(*sys.exc_info(), limit=limit, chain=chain)
        """Like print_exc() but return a string."""
        # return "".join(self.format_exception(*sys.exc_info(), limit=limit, chain=chain))
        return self.format_exception(*sys.exc_info(), limit=limit, chain=chain)

    def log(self, ex: Exception) -> "Span":
        self.error_occurred = True
        self.logs.append(Log(items=[LogItem(key="Traceback", val=str(ex))]))
        return self

    def tag(self, tag: Tag) -> "Span":
        if tag.overridable:
            for i, t in enumerate(self.tags):
                if t.key == tag.key:
                    self.tags[i] = deepcopy(tag)
                    return self

        self.tags.append(deepcopy(tag))

        return self

    def inject(self) -> "Carrier":
        raise RuntimeWarning("只能将上下文载体注入ExitSpan，这可能是代理中的潜在错误，如果遇到这种情况，请在联系fast_tracker团队。 ")

    def extract(self, carrier: "Carrier") -> "Span":
        if carrier is None:
            return self

        self.context.segment.relate(ID(carrier.trace_id))
        self.context._correlation = carrier.correlation_carrier.correlation
        return self

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if isinstance(exc_val, Exception):
            self.raised()
        self.stop()
        if exc_tb is not None:
            return False
        return True


@tostring
class StackedSpan(Span):
    def __init__(self, *args, **kwargs):
        Span.__init__(self, *args, **kwargs)
        self._depth = 0

    def start(self):
        self._depth += 1
        if self._depth == 1:
            Span.start(self)

    def stop(self):
        self._depth -= 1
        if self._depth == 0:
            Span.stop(self)


@tostring
class EntrySpan(StackedSpan):
    def __init__(
        self,
        context: "SpanContext",
        sid: str = "",
        pid: str = "",
        op: str = None,
        peer: str = None,
        component: "Component" = None,
        layer: "Layer" = None,
    ):
        StackedSpan.__init__(
            self,
            context,
            sid,
            pid,
            op,
            peer,
            Kind.Entry,
            component,
            layer,
        )
        self._max_depth = 0

    def start(self):
        StackedSpan.start(self)
        self._max_depth = self._depth
        self.component = 0
        self.layer = Layer.Local
        self.logs = []
        self.tags = []

    def extract(self, carrier: "Carrier") -> "Span":
        Span.extract(self, carrier)

        if carrier is None or not carrier.is_valid:
            return self

        ref = SegmentRef(carrier=carrier)

        if ref not in self.refs:
            self.refs.append(ref)

        return self


@tostring
class ExitSpan(StackedSpan):
    def __init__(
        self,
        context: "SpanContext",
        sid: str = "",
        pid: str = "",
        op: str = None,
        peer: str = None,  # 节点信息 节点的地址和端口
        component: "Component" = None,
        layer: "Layer" = None,
    ):
        StackedSpan.__init__(
            self,
            context,
            sid,
            pid,
            op,
            peer,
            Kind.Exit,
            component,
            layer,
        )

    def inject(self) -> "Carrier":
        return Carrier(
            trace_id=str(self.context.segment.related_traces[0]),
            segment_id=str(self.context.segment.segment_id),
            span_id=str(self.sid),
            service=config.service_name,
            service_instance=config.service_instance,
            endpoint=self.op,
            client_address=self.peer,
            correlation=self.context._correlation,
        )


@tostring
class NoopSpan(Span):
    def __init__(self, context: "SpanContext" = None, kind: "Kind" = None):
        Span.__init__(self, context=context, kind=kind)

    def extract(self, carrier: "Carrier"):
        if carrier is not None:
            self.context._correlation = carrier.correlation_carrier.correlation

    def inject(self) -> "Carrier":
        return Carrier(correlation=self.context._correlation)



#
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from skywalking import agent, config

if __name__ == "__main__":
    config.service_name = "provider"
    config.logging_level = "DEBUG"
    agent.start()

    import json
    import time

    import tornado.ioloop
    import tornado.web

    class MainHandler(tornado.web.RequestHandler):
        def get(self):
            time.sleep(0.5)
            self.write(json.dumps({'song': 'Despacito', 'artist': 'Luis Fonsi'}))

    def make_app():
        return tornado.web.Application([
            (r"/users", MainHandler),
        ])

    app = make_app()
    app.listen(9091, '0.0.0.0')
    tornado.ioloop.IOLoop.current().start()



#
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import time
import requests
from fast_tracker import agent, config
from fast_tracker.trace.ipc.process import FastProcess
import multiprocessing


def post():
    requests.post("http://provider:9091/users")
    time.sleep(3)


if __name__ == '__main__':
    multiprocessing.set_start_method('spawn')
    config.service_name = 'consumer'
    config.logging_level = 'DEBUG'
    config.flask_collect_http_params = True
    agent.start()

    from flask import Flask, jsonify

    app = Flask(__name__)

    @app.route("/users", methods=["POST", "GET"])
    def application():
        p1 = FastProcess(target=post)
        p1.start()
        p1.join()

        res = requests.post("http://provider:9091/users")

        return jsonify(res.json())

    PORT = 9090
    app.run(host='0.0.0.0', port=PORT, debug=False)



#
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from skywalking import config, agent

if __name__ == '__main__':
    config.service_name = 'consumer'
    config.logging_level = 'INFO'
    agent.start()

    import pika

    parameters = (pika.URLParameters("amqp://admin:admin@rabbitmq-server:5672/%2F"))

    connection = pika.BlockingConnection(parameters)

    channel = connection.channel()
    channel.queue_declare("test")
    channel.exchange_declare("test")
    channel.queue_bind(exchange='test', queue="test", routing_key='test')
    for method_frame, properties, body in channel.consume('test'):
        # Display the message parts and acknowledge the message
        print(method_frame, properties, body)
        channel.basic_ack(method_frame.delivery_tag)

        # Escape out of the loop after 10 messages
        if method_frame.delivery_tag == 10:
            break

    try:
        # Loop so we can communicate with RabbitMQ
        connection.ioloop.start()
    except KeyboardInterrupt:
        # Gracefully close the connection
        connection.close()
        # Loop until we're fully closed, will stop on its own
        connection.ioloop.start()



#!/usr/local/bin python3
# -*- coding: utf-8 -*-

"""
    created by FAST-DEV 2021/4/6
"""

from functools import wraps
from typing import List
import inspect

from fast_tracker import Layer, Component
from fast_tracker.trace.context import get_context
from fast_tracker.trace.tags import Tag


def trace(
        op: str = None,
        layer: Layer = Layer.Local,
        component: Component = Component.Unknown,
        tags: List[Tag] = None,
):
    def decorator(func):
        _op = op or func.__name__
        context = get_context()

        span = context.new_local_span(op=_op)
        span.layer = layer
        span.component = component
        [span.tag(tag) for tag in tags or []]

        if inspect.iscoroutinefunction(func):
            @wraps(func)
            async def wrapper(*args, **kwargs):
                with span:
                    return await func(*args, **kwargs)

            return wrapper

        else:
            @wraps(func)
            def wrapper(*args, **kwargs):
                with span:
                    return func(*args, **kwargs)

            return wrapper

    return decorator


def runnable(
        op: str = None,
        layer: Layer = Layer.Local,
        component: Component = Component.Unknown,
        tags: List[Tag] = None,
):
    def decorator(func):
        snapshot = get_context().capture()

        @wraps(func)
        def wrapper(*args, **kwargs):
            _op = op or "Thread/" + func.__name__
            context = get_context()
            with context.new_local_span(op=_op) as span:
                context.continued(snapshot)
                span.layer = layer
                span.component = component
                [span.tag(tag) for tag in tags or []]
                func(*args, **kwargs)

        return wrapper

    return decorator



"""testdj URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/3.1/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""


# from django.contrib import admin
# from django.urls import path
#
# urlpatterns = [
#     path('admin/', admin.site.urls),
# ]
from django.conf.urls import url
from django.urls import re_path, include

from tests.django_project.TestModel import testdb
from . import views

urlpatterns = [
    url(r'^$', views.hello),
    url('hello/', views.hello),
    url('testdb/', testdb.testdb),

    re_path(r'^index/$', views.index, name='index'),
    # re_path(r'^bio/(?P<username>\w+)/$', views.bio, name='bio'),
    # re_path(r'^weblog/', include('blog.urls')),
]


#!/usr/local/bin python3
# -*- coding: utf-8 -*-

"""
    created by FAST-DEV 2021/4/9
"""
import json

import os

from fast_tracker.loggings import logger
from fast_tracker.utils import exceptions, functions


class FastTrackerConfiger:
    @staticmethod
    def _default_config_keys():
        return [
            "Enable",
            "EnvCode",
            "TenantCode",
            "UserCode",
            "ProductCode",
            "AppCode",
            "ServiceName",
            "SocketPath",
            "BufferSize",
            "SocketTimeout",
            "Event",
            "TenantCodeReader",
            "UserCodeReader",
            "CarrierHeader",
        ]

    @staticmethod
    def load_configuration(config_file=None):
        """
        :param config_file: 配置文件地址
        :param log_level:
        :return:
        """
        if not config_file:
            logger.debug("没有代理配置文件")
            raise exceptions.ConfigurationError("没有发现配置文件")
        logger.debug("代理的配置文件是 %s" % config_file)

        try:
            with open(config_file, "r") as fb:
                config_dict = json.load(fb)
                default_config_keys = FastTrackerConfiger._default_config_keys()
                from fast_tracker import config

                if config_dict:
                    for config_key in config_dict.keys():
                        if config_key in default_config_keys:
                            func_name = "set_" + functions.lower_case_name(config_key)
                            # 如果环境变量已经有值，则不再赋值
                            config_name = (
                                "FastTracker_" + config_key if config_key != "SocketPath" else "CollectorAddress"
                            )
                            is_exist = os.getenv(config_name, "None")
                            if is_exist == "None":
                                getattr(config, func_name)(config_dict.get(config_key))

        except Exception as e:
            raise exceptions.ConfigurationError("json格式配置文件格式不合法(不要有注释),解析失败,文件: %s." % config_file)



#!/usr/local/bin python3
# -*- coding: utf-8 -*-

"""
    created by FAST-DEV 2021/4/6
"""

from collections import namedtuple

Tag = namedtuple("Tag", "key val overridable")
Tag.__new__.__defaults__ = (None, None, False)

# http
HttpUrl = "url"
HttpPath = "path"
HttpMethod = "http_method"
HttpStatus = "status_code"
HttpParams = "http_params"

# db
DbType = "db_type"
DbInstance = "db_instance"
DbStatement = "db_statement"
DbSqlParameters = "db_sql_parameters"

# mq
MqType = "mq_broker"
MqBroker = "mq_broker"
MqTopic = "mq_topic"
MqQueue = "mq_queue"

# cache
CacheType = "cache_type"
CacheInstance = "cache_instance"
CacheCommand = "cache_command"



#!/usr/local/bin python3
# -*- coding: utf-8 -*-

"""
    created by FAST-DEV 2021/4/6
"""

from urllib.request import Request

from fast_tracker import Layer, ComponentType
from fast_tracker.trace import tags
from fast_tracker.trace.context import get_context
from fast_tracker.trace.tags import Tag
from fast_tracker.utils import functions


def install():
    import socket
    from urllib.request import OpenerDirector
    from urllib.error import HTTPError
    from urllib.parse import urlsplit

    _open = OpenerDirector.open

    def _fast_open(this: OpenerDirector, fullurl, data=None, timeout=socket._GLOBAL_DEFAULT_TIMEOUT):
        if isinstance(fullurl, str):
            fullurl = Request(fullurl, data)

        context = get_context()
        url = fullurl.selector.split("?")[0] if fullurl.selector else "/"
        with context.new_exit_span(op=url, peer=fullurl.host) as span:
            carrier = span.inject()
            span.layer = Layer.Http
            span.component = ComponentType.General
            code = None

            [fullurl.add_header(item.key, item.val) for item in carrier]

            try:
                res = _open(this, fullurl, data, timeout)
                code = res.code
            except HTTPError as e:
                code = e.code
                raise
            finally:  # we do this here because it may change in _open()
                span.tag(Tag(key=tags.HttpMethod, val=fullurl.get_method()))
                span.tag(Tag(key=tags.HttpUrl, val=fullurl.full_url))
                # HttpPath 只需要获取PATH_INFO信息即可，不需要？后的内容
                # urlsplit 与 urlparse 的对比参考 fast_urllib3.py
                split_url = urlsplit(fullurl.selector)
                span.tag(Tag(key=tags.HttpPath, val=split_url.path))

                if code is not None:
                    span.tag(Tag(key=tags.HttpStatus, val=code, overridable=True))

                    if code >= 400:
                        span.error_occurred = True

            functions.log("urllib_request span is: %r", span)

            return res

    OpenerDirector.open = _fast_open



#
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import time

import aiohttp
from aiohttp import web

from fast_tracker import agent
from fast_tracker import config


async def handle(request):
    name = request.match_info.get('name', "Anonymous")

    async with aiohttp.ClientSession() as session:
        async with session.get('http://user:pass@provider:9091/{name}') as response:
            time.sleep(.5)
            json = await response.json()
            return web.Response(text=str(json))


app = web.Application()
app.add_routes([web.get('/', handle), web.get('/{name}', handle)])

if __name__ == '__main__':
    config.service_name = 'consumer'
    config.logging_level = 'DEBUG'
    agent.start()

    web.run_app(app, port=9090)



#
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import time

from skywalking import agent, config
from flask import Flask, jsonify
from pymongo import MongoClient


config.service_name = "provider"
config.logging_level = "DEBUG"
config.pymongo_trace_parameters = True
agent.start()


client = MongoClient('mongodb://mongo:27017/')
db = client['test-database']
collection = db['test-collection']


app = Flask(__name__)


@app.route("/insert_many", methods=["GET"])
def test_insert_many():
    time.sleep(0.5)
    new_posts = [{"song": "Despacito"},
                 {"artist": "Luis Fonsi"}]
    result = collection.insert_many(new_posts)
    return jsonify({"ok": result.acknowledged})


@app.route("/find_one", methods=["GET"])
def test_find_one():
    time.sleep(0.5)
    result = collection.find_one({"song": "Despacito"})
    # have to get the result and use it. if not lint will report error
    print(result)
    return jsonify({"song": "Despacito"})


@app.route("/delete_one", methods=["GET"])
def test_delete_one():
    time.sleep(0.5)
    result = collection.delete_one({"song": "Despacito"})
    return jsonify({"ok": result.acknowledged})


if __name__ == '__main__':
    PORT = 9091
    app.run(host="0.0.0.0", port=PORT, debug=True)



#
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from typing import Callable

import pytest
import requests

from tests.plugin.base import TestPluginBase


@pytest.fixture
def prepare():
    # type: () -> Callable
    return lambda *_: requests.post('http://0.0.0.0:9090')


class TestPlugin(TestPluginBase):
    def test_plugin(self, docker_compose, version):
        self.validate()



# -*- coding: utf-8 -*-
# print("asdfasddddddd")
#!/usr/local/bin python3
# -*- coding: utf-8 -*-

"""
    created by FAST-DEV 2021/4/6
"""
from __future__ import print_function

from fast_tracker.admin import command, usage


@command("run-program", "...", """执行命令行来启动应用,但它会强制初始化代理. 环境变量FastTrackerConfigPath可以提供代理配置文件的路径,日志文件的详情由FAST_LOG提供""")
def run_program(args):
    import os
    import sys
    import time

    if len(args) == 0:
        usage("run-program")
        sys.exit(1)

    # startup_debug = os.environ.get("FAST_STARTUP_DEBUG", "off").lower() in ("on", "true", "1")

    def log_message(text, *args):
        # if startup_debug:
        text = text % args
        timestamp = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
        print("FAST: %s (%d) - %s" % (timestamp, os.getpid(), text))

    log_message("FAST Admin Script (%s)", __file__)

    log_message("working_directory = %r", os.getcwd())
    log_message("current_command = %r", sys.argv)

    log_message("sys.prefix = %r", os.path.normpath(sys.prefix))

    try:
        log_message("sys.real_prefix = %r", sys.real_prefix)
    except AttributeError:
        pass

    log_message("sys.version_info = %r", sys.version_info)
    log_message("sys.executable = %r", sys.executable)
    log_message("sys.flags = %r", sys.flags)
    log_message("sys.path = %r", sys.path)

    for name in sorted(os.environ.keys()):
        if name.startswith("FAST_") or name.startswith("PYTHON"):
            log_message("%s = %r", name, os.environ.get(name))

    from fast_tracker import __file__ as root_directory

    root_directory = os.path.dirname(root_directory)
    boot_directory = os.path.join(root_directory, "bootstrap")

    log_message("root_directory = %r", root_directory)
    log_message("boot_directory = %r", boot_directory)

    python_path = boot_directory

    if "PYTHONPATH" in os.environ:
        path = os.environ["PYTHONPATH"].split(os.path.pathsep)
        if boot_directory not in path:
            python_path = "%s%s%s" % (boot_directory, os.path.pathsep, os.environ["PYTHONPATH"])

    os.environ["PYTHONPATH"] = python_path

    os.environ["FAST_ADMIN_COMMAND"] = repr(sys.argv)

    os.environ["FAST_PYTHON_PREFIX"] = os.path.realpath(os.path.normpath(sys.prefix))
    os.environ["FAST_PYTHON_VERSION"] = ".".join(map(str, sys.version_info[:2]))

    program_exe_path = args[0]

    if not os.path.dirname(program_exe_path):
        # 如果不是绝对路径或者相对路径，则去系统的环境变量里找Python程序的执行路径
        program_search_path = os.environ.get("PATH", "").split(os.path.pathsep)
        for path in program_search_path:
            path = os.path.join(path, program_exe_path)
            if os.path.exists(path) and os.access(path, os.X_OK):
                program_exe_path = path
                break

    log_message("program_exe_path = %r", program_exe_path)
    log_message("execl_arguments = %r", [program_exe_path] + args)

    os.execl(program_exe_path, *args)




#
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from urllib import request

from skywalking import agent, config

if __name__ == '__main__':
    config.service_name = 'consumer'
    agent.start()

    import socketserver
    from http.server import BaseHTTPRequestHandler

    class SimpleHTTPRequestHandler(BaseHTTPRequestHandler):
        def do_POST(self):
            self.send_response(200)
            self.send_header('Content-Type', 'application/json; charset=utf-8')
            self.end_headers()

            data = '{"name": "whatever"}'.encode('utf8')
            req = request.Request('http://provider:9091/users')
            req.add_header('Content-Type', 'application/json; charset=utf-8')
            req.add_header('Content-Length', str(len(data)))
            with request.urlopen(req, data):
                self.wfile.write(data)

    PORT = 9090
    Handler = SimpleHTTPRequestHandler

    with socketserver.TCPServer(("", PORT), Handler) as httpd:
        print("serving at port", PORT)
        httpd.serve_forever()



#
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import time

from skywalking import agent, config

if __name__ == '__main__':
    config.service_name = 'provider'
    config.logging_level = 'DEBUG'
    agent.start()

    from flask import Flask, jsonify
    import redis

    app = Flask(__name__)

    @app.route("/users", methods=["POST", "GET"])
    def application():
        time.sleep(0.5)

        r = redis.StrictRedis(host='redis', port=6379, db=0)

        r.set('foo', 'bar')
        r.get('foo')

        return jsonify({"song": "Despacito", "artist": "Luis Fonsi"})

    PORT = 9091
    app.run(host='0.0.0.0', port=PORT, debug=True)



#
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from skywalking import agent, config

if __name__ == '__main__':
    config.service_name = 'producer'
    config.logging_level = 'INFO'
    agent.start()

    from flask import Flask, jsonify
    from kafka import KafkaProducer

    app = Flask(__name__)
    producer = KafkaProducer(bootstrap_servers=['kafka-server:9092'], api_version=(1, 0, 1))

    @app.route("/users", methods=["POST", "GET"])
    def application():
        producer.send('skywalking', b'some_message_bytes')

        return jsonify({"song": "Despacito", "artist": "Luis Fonsi"})

    PORT = 9090
    app.run(host='0.0.0.0', port=PORT, debug=True)



#
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# if __name__ == '__main__':
#     import sys
#     sys.path.append(sys.path[0]+'/../../../..')

import sys
import time

from fast_tracker import agent, config

from django.conf import settings
from django.conf.urls import url
from django.http import JsonResponse



config.service_name = "provider"
config.logging_level = "DEBUG"
agent.start()


settings.configure(
    DEBUG=True,
    ROOT_URLCONF=__name__,
    ALLOWED_HOSTS=['*'],
)


def index(request):
    time.sleep(0.5)
    return JsonResponse({"song": "Despacito", "artist": "Luis Fonsi"})


urlpatterns = (
    url("users", index),
)


if __name__ == "__main__":
    from django.core.management import execute_from_command_line

    execute_from_command_line(sys.argv)



#
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from wsgiref.simple_server import make_server

from pyramid.config import Configurator
from pyramid.response import Response

from skywalking import agent, config


def index(request):
    return Response('Hello World!')


def error(request):
    raise Exception('Error!')


if __name__ == '__main__':
    config.service_name = 'provider'
    config.logging_level = 'DEBUG'
    agent.start()

    with Configurator() as config:
        config.add_route('pyramid', '/pyramid')
        config.add_route('error', '/error')
        config.add_view(index, route_name='pyramid')
        config.add_view(error, route_name='error')

        app = config.make_wsgi_app()

    server = make_server('0.0.0.0', 9091, app)

    server.serve_forever()



#!/usr/local/bin python3
# -*- coding: utf-8 -*-

"""
    created by iprobeyang@gmail.com 2021/4/13
"""

from django.http import HttpResponse


def hello(request):
    from django.shortcuts import redirect
    # redirect(index(request))
    redirect("http://localhost:8000/index")
    return HttpResponse("Hello world ! ")


def index(request):

    return HttpResponse("index")


#!/usr/local/bin python3
# -*- coding: utf-8 -*-

"""
    created by FAST-DEV 2021/4/6
"""

from fast_tracker import Layer, ComponentType
from fast_tracker.trace import tags
from fast_tracker.trace.context import get_context
from fast_tracker.trace.tags import Tag
from fast_tracker.utils import functions


def install():
    from pymysql.cursors import Cursor

    _execute = Cursor.execute

    def _fast_execute(this: Cursor, query, args=None):
        peer = "%s:%s" % (this.connection.host, this.connection.port)

        context = get_context()
        with context.new_exit_span(op="Mysql/PyMsql/execute", peer=peer) as span:
            span.layer = Layer.Database
            span.component = ComponentType.PyMysql
            res = _execute(this, query, args)

            span.tag(Tag(key=tags.DbType, val="mysql"))
            span.tag(Tag(key=tags.DbInstance, val=(this.connection.db or b'').decode("utf-8")))
            span.tag(Tag(key=tags.DbStatement, val=query))

            parameter = ",".join([str(arg) for arg in args]) if args else ''
            span.tag(Tag(key=tags.DbSqlParameters, val='[' + parameter + ']'))

            functions.log("mysql span is: %r", span)

            return res

    Cursor.execute = _fast_execute



#!/usr/local/bin python3
# -*- coding: utf-8 -*-

"""
    created by FAST-DEV 2021/4/6
"""

from fast_tracker import agent, config
from fast_tracker.trace import ID
from fast_tracker.trace.carrier import Carrier
from fast_tracker.trace.segment import Segment, SegmentRef
from fast_tracker.trace.snapshot import Snapshot
from fast_tracker.trace.span import Span, Kind, NoopSpan, EntrySpan, ExitSpan
from fast_tracker.utils.counter import SpanCounter

try:  # attempt to use async-local instead of thread-local context and spans
    import contextvars

    __local = contextvars.ContextVar("local")
    __spans = contextvars.ContextVar("spans")  # this needs to be a per-task variable, can't be part of __local
    _spans = __spans.get
    _spans_set = __spans.set  # pyre-ignore

    class AsyncLocal:
        pass

    def _local():
        try:
            return __local.get()

        except LookupError:
            local = AsyncLocal()
            __local.set(local)

            return local

    def _spans_dup():
        spans = __spans.get()[:]
        __spans.set(spans)

        return spans


except ImportError:
    import threading

    __local = threading.local()

    def _local():
        return __local

    def _spans():
        return __local.spans

    def _spans_set(spans):
        __local.spans = spans

    _spans_dup = _spans


class SpanContext(object):
    def __init__(self):
        self.segment = Segment()  # type: Segment
        self._sid = SpanCounter()
        self._correlation = {}  # type: dict
        self._nspans = 0

    def new_local_span(self, op: str) -> Span:
        span = self.ignore_check(op, Kind.Local)
        if span is not None:
            return span

        spans = _spans_dup()
        parent = spans[-1] if spans else None  # type: Span

        return Span(
            context=self,
            sid=self._sid.next(),
            pid=parent.sid if parent else "",
            op=op,
            kind=Kind.Local,
        )

    def new_entry_span(self, op: str, carrier: "Carrier" = None) -> Span:
        span = self.ignore_check(op, Kind.Entry)
        if span is not None:
            return span

        spans = _spans_dup()
        parent = spans[-1] if spans else None  # type: Span

        parent_id = ""
        if parent:
            parent_id = parent.sid
        elif carrier.span_id:  # 当跨服务调用的时候，carrier.span_id等都有值，span_id为上一个服务调用此服务的span_id
            parent_id = carrier.span_id

        span = (
            parent
            if parent is not None and parent.kind.is_entry
            else EntrySpan(
                context=self,
                sid=self._sid.next(),
                pid=parent_id,
            )
        )
        span.op = op

        if carrier is not None and carrier.is_valid:
            span.extract(carrier=carrier)

        return span

    def new_exit_span(self, op: str, peer: str) -> Span:
        span = self.ignore_check(op, Kind.Exit)
        if span is not None:
            return span

        spans = _spans_dup()
        parent = spans[-1] if spans else None  # type: Span

        span = (
            parent
            if parent is not None and parent.kind.is_exit
            else ExitSpan(
                context=self,
                sid=self._sid.next(),
                pid=parent.sid if parent else "",
                op=op,
                peer=peer,
            )
        )

        return span

    def ignore_check(self, op: str, kind: Kind):
        if config.RE_IGNORE_PATH.match(op):
            return NoopSpan(
                context=NoopContext(),
                kind=kind,
            )

        return None

    def start(self, span: Span):
        self._nspans += 1
        spans = _spans()
        if span not in spans:
            spans.append(span)

    def stop(self, span: Span) -> bool:
        spans = _spans()
        span.finish(self.segment)
        del spans[spans.index(span)]

        self._nspans -= 1
        if self._nspans == 0:
            _local().context = None
            agent.archive(self.segment)
            return True

        return False

    def active_span(self):
        spans = _spans()
        if spans:
            return spans[len(spans) - 1]

        return None

    def get_correlation(self, key):
        if key in self._correlation:
            return self._correlation[key]
        return None

    def put_correlation(self, key, value):
        if key is None:
            return
        if value is None:
            self._correlation.pop(key, value)
            return
        if len(value) > config.correlation_value_max_length:
            return
        if len(self._correlation) > config.correlation_element_max_number:
            return

        self._correlation[key] = value

    def capture(self):
        spans = _spans()
        if len(spans) == 0:
            return None

        return Snapshot(
            segment_id=str(self.segment.segment_id),
            span_id=self.active_span().sid,
            trace_id=self.segment.related_traces[0],
            endpoint=spans[0].op,
            correlation=self._correlation,
        )

    def continued(self, snapshot: "Snapshot"):
        if snapshot is None:
            return None
        if not snapshot.is_from_current(self) and snapshot.is_valid():
            ref = SegmentRef.build_ref(snapshot)
            span = self.active_span()
            span.refs.append(ref)
            self.segment.relate(ID(ref.trace_id))
            self._correlation.update(snapshot.correlation)


class NoopContext(SpanContext):
    def __init__(self):
        super().__init__()
        self._depth = 0
        self._noop_span = NoopSpan(self, kind=Kind.Local)
        self.correlation = {}  # type: dict

    def new_local_span(self, op: str) -> Span:
        return self._noop_span

    def new_entry_span(self, op: str, carrier: "Carrier" = None) -> Span:
        if carrier is not None:
            self._noop_span.extract(carrier)
        return self._noop_span

    def new_exit_span(self, op: str, peer: str) -> Span:
        return self._noop_span

    def start(self, span: Span):
        self._depth += 1

    def stop(self, span: Span) -> bool:
        self._depth -= 1
        return self._depth == 0

    def active_span(self):
        return self._noop_span

    def capture(self):
        return Snapshot(
            segment_id=None,
            span_id="",
            trace_id=None,
            endpoint=None,
            correlation=self._correlation,
        )

    def continued(self, snapshot: "Snapshot"):
        self._correlation.update(snapshot.correlation)


def get_context() -> SpanContext:
    local = _local()
    context = getattr(local, "context", False)

    if not context:
        context = local.context = SpanContext() if agent.connected() else NoopContext()
        _spans_set([])  # XXX would be better in SpanContext.__init__() but for some reason doesn't work there

    return context



#
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import inspect
import os
import sys
import time
from abc import ABC
from difflib import Differ
from os.path import dirname

import requests
import yaml
from requests import Response

try:
    from yaml import CSafeLoader as Loader
except ImportError:
    from yaml import SafeLoader as Loader


class TestPluginBase(ABC):
    def validate(self, expected_file_name=None):
        # type: (str) -> Response

        if expected_file_name is None:
            expected_file_name = os.path.join(dirname(inspect.getfile(self.__class__)), 'expected.data.yml')

        time.sleep(10)

        with open(expected_file_name) as expected_data_file:
            expected_data = os.linesep.join(expected_data_file.readlines())

            response = requests.post(url='http://0.0.0.0:12800/dataValidate', data=expected_data)

            if response.status_code != 200:
                res = requests.get('http://0.0.0.0:12800/receiveData')

                actual_data = yaml.dump(yaml.load(res.content, Loader=Loader))

                differ = Differ()
                diff_list = list(differ.compare(
                    actual_data.splitlines(keepends=True),
                    yaml.dump(yaml.load(expected_data, Loader=Loader)).splitlines(keepends=True)
                ))

                print('diff list: ')

                sys.stdout.writelines(diff_list)

            assert response.status_code == 200

            return response



#!/usr/local/bin python3
# -*- coding: utf-8 -*-

"""
    created by FAST-DEV 2021/4/6
"""

import base64


def tostring(cls):
    def __str__(self):
        return '%s@%s[%s]' % (
            type(self).__name__, id(self),
            ', '.join(
                '%s=%s' % (k, str(v)) for (k, v) in vars(self).items()
            )
        )

    cls.__str__ = __str__
    return cls


def b64encode(s: str = '') -> str:
    return base64.b64encode(s.encode('utf8')).decode('utf8')


def b64decode(s: str = '') -> str:
    return base64.b64decode(s).decode('utf8')



#!/usr/local/bin python3
# -*- coding: utf-8 -*-

"""
    created by FAST-DEV 2021/4/6
"""

from fast_tracker import Layer, ComponentType
from fast_tracker.trace import tags
from fast_tracker.trace.carrier import Carrier
from fast_tracker.trace.context import get_context
from fast_tracker.trace.tags import Tag


def install():
    from kafka import KafkaProducer
    from kafka import KafkaConsumer

    _send = KafkaProducer.send
    __poll_once = KafkaConsumer._poll_once
    KafkaProducer.send = _fast_send_func(_send)
    KafkaConsumer._poll_once = _fast__poll_once_func(__poll_once)


def _fast__poll_once_func(__poll_once):
    def _fast__poll_once(this, timeout_ms, max_records, update_offsets=True):
        res = __poll_once(this, timeout_ms, max_records, update_offsets=update_offsets)
        if res:
            brokers = ";".join(this.config["bootstrap_servers"])
            context = get_context()
            topics = ";".join(this._subscription.subscription or [t.topic for t in this._subscription._user_assignment])

            with context.new_entry_span(op="Kafka/" + topics + "/Consumer/" + (this.config["group_id"] or "")) as span:
                for consumerRecords in res.values():
                    for record in consumerRecords:
                        carrier = Carrier()
                        for item in carrier:
                            for header in record.headers:
                                if item.key == header[0]:
                                    item.val = str(header[1])

                        span.extract(carrier)
                    span.tag(Tag(key=tags.MqType, val=ComponentType.Kafka))
                    span.tag(Tag(key=tags.MqBroker, val=brokers))
                    span.tag(Tag(key=tags.MqTopic, val=topics))
                    span.layer = Layer.MQ
                    span.component = ComponentType.KafkaConsumer

        return res

    return _fast__poll_once


def _fast_send_func(_send):
    def _fast_send(this, topic, value=None, key=None, headers=None, partition=None, timestamp_ms=None):
        peer = ";".join(this.config["bootstrap_servers"])
        context = get_context()
        with context.new_exit_span(op="Kafka/" + topic + "/Producer" or "/", peer=peer) as span:
            carrier = span.inject()
            span.layer = Layer.MQ
            span.component = ComponentType.KafkaProducer

            if headers is None:
                headers = []
            for item in carrier:
                headers.append((item.key, item.val.encode("utf-8")))

            res = _send(
                this, topic, value=value, key=key, headers=headers, partition=partition, timestamp_ms=timestamp_ms
            )
            span.tag(Tag(key=tags.MqType, val=ComponentType.Kafka))
            span.tag(Tag(key=tags.MqBroker, val=peer))
            span.tag(Tag(key=tags.MqTopic, val=topic))

            return res

    return _fast_send
